use detee_shared::general_proto::brain_general_cli_server::BrainGeneralCliServer;
use detee_shared::vm_proto::brain_vm_cli_server::BrainVmCliServer;
use surreal_brain::constants::{
    BRAIN_GRPC_ADDR, CERT_KEY_PATH, CERT_PATH, DB_ADDRESS, DB_NAME, DB_NS,
};
use surreal_brain::db;
use surreal_brain::grpc::BrainGeneralCliForReal;
use surreal_brain::grpc::BrainVmCliForReal;
use tonic::transport::{Identity, Server, ServerTlsConfig};

#[tokio::main]
async fn main() {
    env_logger::builder().filter_level(log::LevelFilter::Debug).init();
    db::init(DB_ADDRESS, DB_NS, DB_NAME).await.unwrap();
    let addr = BRAIN_GRPC_ADDR.parse().unwrap();

    let snp_cli_server = BrainVmCliServer::new(BrainVmCliForReal {});
    let general_service_server = BrainGeneralCliServer::new(BrainGeneralCliForReal {});

    let cert = std::fs::read_to_string(CERT_PATH).unwrap();
    let key = std::fs::read_to_string(CERT_KEY_PATH).unwrap();

    let identity = Identity::from_pem(cert, key);

    Server::builder()
        .tls_config(ServerTlsConfig::new().identity(identity))
        .unwrap()
        .add_service(snp_cli_server)
        .add_service(general_service_server)
        .serve(addr)
        .await
        .unwrap();
}
